@@ -29,16 +29,15 @@ import org.apache.catalina.Globals;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Service;
-import org.apache.catalina.core.AprStatus;
 import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.coyote.AbstractProtocol;
 import org.apache.coyote.Adapter;
 import org.apache.coyote.ProtocolHandler;
 import org.apache.coyote.UpgradeProtocol;
 import org.apache.coyote.http11.AbstractHttp11JsseProtocol;
-import org.apache.coyote.http11.AbstractHttp11Protocol;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.jni.AprStatus;
 import org.apache.tomcat.util.IntrospectionUtils;
 import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.CharsetUtil;
@@ -62,6 +61,22 @@ public class Connector extends LifecycleMBeanBase {
 
     public static final String INTERNAL_EXECUTOR_NAME = "Internal";
 
+    private static final boolean aprStatusPresent;
+
+    static {
+        /*
+         * The AprStatus class has to be in the org.apache.tomcat.jni package so it can be referenced by the OpenSSL
+         * clean-up code to avoid a race condition on shutdown between the AprLifecycleListener shutting down the Tomcat
+         * Native library along with any remaining open connections and the OpenSSL clean-up code shutting down an
+         * individual connection that can trigger a JVM crash.
+         *
+         * In some deployment scenarios AprStatus is not present - e.g. because tomcat-jni.jar is not present. To avoid
+         * a CNFE in this class on Connector initialisation when AprStatus is not present - and ugly work-arounds that
+         * try loading the class and catching the exception - use getResource() to see if AprStatus is present.
+         */
+        aprStatusPresent =
+                (Connector.class.getClassLoader().getResource("org/apache/tomcat/jni/AprStatus.class") != null);
+    }
 
     // ------------------------------------------------------------ Constructor
 
@@ -1047,25 +1062,21 @@ public class Connector extends LifecycleMBeanBase {
             setParseBodyMethods(getParseBodyMethods());
         }
 
-        if (JreCompat.isJre22Available() && OpenSSLStatus.getUseOpenSSL() && OpenSSLStatus.isAvailable() &&
-                protocolHandler instanceof AbstractHttp11Protocol) {
-            // Use FFM and OpenSSL if available
+        if (protocolHandler instanceof AbstractHttp11JsseProtocol) {
             AbstractHttp11JsseProtocol<?> jsseProtocolHandler = (AbstractHttp11JsseProtocol<?>) protocolHandler;
             if (jsseProtocolHandler.isSSLEnabled() && jsseProtocolHandler.getSslImplementationName() == null) {
-                // OpenSSL is compatible with the JSSE configuration, so use it if it is available
-                jsseProtocolHandler
-                        .setSslImplementationName("org.apache.tomcat.util.net.openssl.panama.OpenSSLImplementation");
-            }
-        } else if (AprStatus.isAprAvailable() && AprStatus.getUseOpenSSL() &&
-                protocolHandler instanceof AbstractHttp11Protocol) {
-            // Use tomcat-native and OpenSSL otherwise, if available
-            AbstractHttp11JsseProtocol<?> jsseProtocolHandler = (AbstractHttp11JsseProtocol<?>) protocolHandler;
-            if (jsseProtocolHandler.isSSLEnabled() && jsseProtocolHandler.getSslImplementationName() == null) {
-                // OpenSSL is compatible with the JSSE configuration, so use it if APR is available
-                jsseProtocolHandler.setSslImplementationName(OpenSSLImplementation.class.getName());
+                // If SSL is enabled and a specific implementation isn't specified, select the correct default.
+                if (JreCompat.isJre22Available() && OpenSSLStatus.getUseOpenSSL() && OpenSSLStatus.isAvailable()) {
+                    // Use FFM and OpenSSL if available
+                    jsseProtocolHandler
+                            .setSslImplementationName("org.apache.tomcat.util.net.openssl.panama.OpenSSLImplementation");
+                } else if (aprStatusPresent && AprStatus.isAprAvailable() && AprStatus.getUseOpenSSL()) {
+                    // Use tomcat-native and OpenSSL otherwise, if available
+                    jsseProtocolHandler.setSslImplementationName(OpenSSLImplementation.class.getName());
+                }
+                // Otherwise the default JSSE will be used
             }
         }
-        // Otherwise the default JSSE will be used
 
         try {
             protocolHandler.init();
