@@ -318,13 +318,13 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader {
                 MigrationUtil.addJakartaEETransformer(classLoader, getJakartaConverter());
             }
 
-            // Configure our repositories
-            setClassPath();
-
             setPermissions();
 
             classLoader.start();
 
+            // Configure our repositories
+            setClassPath();
+
             String contextName = context.getName();
             if (!contextName.startsWith("/")) {
                 contextName = "/" + contextName;
@@ -511,8 +511,9 @@ public class WebappLoader extends LifecycleMBeanBase implements Loader {
                     try {
                         File f = new File(url.toURI());
                         repository = f.getAbsolutePath();
-                    } catch (URISyntaxException e) {
+                    } catch (URISyntaxException | IllegalArgumentException e) {
                         // Can't convert from URL to URI. Treat as non-file URL and skip.
+                        continue;
                     }
                 } else {
                     continue;
