@@ -24,12 +24,7 @@ import java.text.DecimalFormat;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
- * Implements the Server-side #fsize command
- *
- * @author Bip Thelin
- * @author Paul Speed
- * @author Dan Sandberg
- * @author David Becker
+ * Implements the Server-side #fsize command.
  */
 public final class SSIFsize implements SSICommand {
     private static final StringManager sm = StringManager.getManager(SSIFsize.class);
@@ -66,23 +61,11 @@ public final class SSIFsize implements SSICommand {
     }
 
 
-    public String repeat(char aChar, int numChars) {
-        if (numChars < 0) {
-            throw new IllegalArgumentException(sm.getString("ssiFsize.invalidNumChars"));
-        }
-        StringBuilder buf = new StringBuilder();
-        for (int i = 0; i < numChars; i++) {
-            buf.append(aChar);
-        }
-        return buf.toString();
-    }
-
-
     public String padLeft(String str, int maxChars) {
         String result = str;
         int charsToAdd = maxChars - str.length();
         if (charsToAdd > 0) {
-            result = repeat(' ', charsToAdd) + str;
+            result = " ".repeat(charsToAdd) + str;
         }
         return result;
     }
