@@ -32,6 +32,7 @@ import java.security.cert.Certificate;
 import java.text.Collator;
 import java.util.Arrays;
 import java.util.Locale;
+import java.util.Objects;
 import java.util.jar.JarFile;
 import java.util.jar.Manifest;
 
@@ -295,7 +296,7 @@ public class CachedResource implements WebResource {
         if (cachedStrongETag == null) {
             byte[] buf = getContent();
             if (buf != null) {
-                buf = ConcurrentMessageDigest.digest("SHA-1", buf);
+                buf = ConcurrentMessageDigest.digestSHA256(buf);
                 cachedStrongETag = "\"" + HexUtils.toHexString(buf) + "\"";
             } else {
                 cachedStrongETag = webResource.getStrongETag();
@@ -595,7 +596,7 @@ public class CachedResource implements WebResource {
         @Override
         public String getContentType() {
             // "content/unknown" is the value used by sun.net.www.URLConnection. It is used here for consistency.
-            return (getResource().getMimeType() != null) ? getResource().getMimeType() : "content/unknown";
+            return Objects.requireNonNullElse(getResource().getMimeType(), "content/unknown");
         }
     }
 
