@@ -24,13 +24,13 @@ import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 
-import javax.servlet.RequestDispatcher;
-import javax.servlet.ServletConfig;
-import javax.servlet.ServletContext;
-import javax.servlet.ServletException;
-import javax.servlet.http.HttpServlet;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
+import jakarta.servlet.RequestDispatcher;
+import jakarta.servlet.ServletConfig;
+import jakarta.servlet.ServletContext;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.http.HttpServlet;
+import jakarta.servlet.http.HttpServletRequest;
+import jakarta.servlet.http.HttpServletResponse;
 
 import org.apache.jasper.Constants;
 import org.apache.jasper.EmbeddedServletOptions;
@@ -48,12 +48,6 @@ import org.apache.tomcat.util.security.Escape;
  * The Jasper JSP engine. The servlet container is responsible for providing a URLClassLoader for the web application
  * context Jasper is being used in. Jasper will try to get the Tomcat ServletContext attribute for its ServletContext
  * class loader, if that fails, it uses the parent class loader. In either case, it must be a URLClassLoader.
- *
- * @author Anil K. Vijendran
- * @author Harish Prabandham
- * @author Remy Maucherat
- * @author Kin-man Chung
- * @author Glenn Nielsen
  */
 public class JspServlet extends HttpServlet implements PeriodicEventListener {
 
@@ -218,16 +212,17 @@ public class JspServlet extends HttpServlet implements PeriodicEventListener {
      */
     boolean preCompile(HttpServletRequest request) throws ServletException {
 
+        String precompileParameter = rctxt.getOptions().getJspPrecompilationQueryParameter();
         String queryString = request.getQueryString();
         if (queryString == null) {
             return false;
         }
-        int start = queryString.indexOf(Constants.PRECOMPILE);
+        int start = queryString.indexOf(precompileParameter);
         if (start < 0) {
             return false;
         }
-        queryString = queryString.substring(start + Constants.PRECOMPILE.length());
-        if (queryString.length() == 0) {
+        queryString = queryString.substring(start + precompileParameter.length());
+        if (queryString.isEmpty()) {
             return true; // ?jsp_precompile
         }
         if (queryString.startsWith("&")) {
@@ -253,7 +248,7 @@ public class JspServlet extends HttpServlet implements PeriodicEventListener {
             return true; // ?jsp_precompile=false
         } else {
             throw new ServletException(
-                    Localizer.getMessage("jsp.error.precompilation.parameter", Constants.PRECOMPILE, value));
+                    Localizer.getMessage("jsp.error.precompilation.parameter", precompileParameter, value));
         }
 
     }
@@ -273,7 +268,7 @@ public class JspServlet extends HttpServlet implements PeriodicEventListener {
             if (jspUri != null) {
                 /*
                  * Requested JSP has been target of RequestDispatcher.include(). Its path is assembled from the relevant
-                 * javax.servlet.include.* request attributes
+                 * jakarta.servlet.include.* request attributes
                  */
                 String pathInfo = (String) request.getAttribute(RequestDispatcher.INCLUDE_PATH_INFO);
                 if (pathInfo != null) {
