@@ -1386,7 +1386,7 @@ public class Digester extends DefaultHandler2 {
                         }
                     } catch (URISyntaxException e) {
                         if (log.isDebugEnabled()) {
-                            log.debug(sm.getString("digester.invalidURI", baseURI, systemId));
+                            log.debug(sm.getString("digester.invalidURI", baseURI, systemId), e);
                         }
                     }
                 }
