@@ -86,6 +86,8 @@ public abstract class SocketWrapperBase<E> {
     protected int remotePort = -1;
     protected volatile ServletConnection servletConnection = null;
 
+    protected String sniHostName = null;
+
     /**
      * Used to record the first IOException that occurs during non-blocking read/writes that can't be usefully
      * propagated up the stack since there is no user code or appropriate container code in the stack to handle it.
@@ -209,6 +211,20 @@ public abstract class SocketWrapperBase<E> {
         this.negotiatedProtocol = negotiatedProtocol;
     }
 
+    /**
+     * @return the sniHostName
+     */
+    public String getSniHostName() {
+        return this.sniHostName;
+    }
+
+    /**
+     * @param sniHostName the SNI host name to set
+     */
+    public void setSniHostName(String sniHostName) {
+        this.sniHostName = sniHostName;
+    }
+
     /**
      * Set the timeout for reading. Values of zero or less will be changed to -1.
      *
@@ -421,10 +437,10 @@ public abstract class SocketWrapperBase<E> {
         if (closed.compareAndSet(false, true)) {
             try {
                 getEndpoint().getHandler().release(this);
-            } catch (Throwable e) {
-                ExceptionUtils.handleThrowable(e);
+            } catch (Throwable t) {
+                ExceptionUtils.handleThrowable(t);
                 if (log.isDebugEnabled()) {
-                    log.error(sm.getString("endpoint.debug.handlerRelease"), e);
+                    log.error(sm.getString("endpoint.debug.handlerRelease"), t);
                 }
             } finally {
                 getEndpoint().countDownConnection();
@@ -1360,7 +1376,13 @@ public abstract class SocketWrapperBase<E> {
             synchronized (state) {
                 if (state.state == CompletionState.PENDING) {
                     try {
-                        state.wait(unit.toMillis(timeout));
+                        long timeoutExpiry = System.nanoTime() + unit.toNanos(timeout);
+                        long timeoutMillis = unit.toMillis(timeout);
+                         // Spurious wake-ups are possible. Keep waiting until state changes or timeout expires.
+                        while (state.state == CompletionState.PENDING && timeoutMillis > 0) {
+                            state.wait(unit.toMillis(timeout));
+                            timeoutMillis = (timeoutExpiry - System.nanoTime()) / 1_000_000;
+                        }
                         if (state.state == CompletionState.PENDING) {
                             if (handler != null && state.callHandler.compareAndSet(true, false)) {
                                 handler.failed(new SocketTimeoutException(getTimeoutMsg(read)), attachment);
