@@ -21,16 +21,11 @@ import java.io.IOException;
 import java.security.Principal;
 import java.util.ArrayList;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicInteger;
 
 import jakarta.servlet.Servlet;
-import jakarta.servlet.ServletConfig;
 import jakarta.servlet.ServletContainerInitializer;
 import jakarta.servlet.ServletContext;
 import jakarta.servlet.ServletException;
@@ -55,6 +50,7 @@ import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.descriptor.web.LoginConfig;
+import org.apache.tomcat.util.http.Method;
 
 public class TestStandardWrapper extends TomcatBaseTest {
 
@@ -65,26 +61,22 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
     @Test
     public void testSecurityAnnotationsSubclass1() throws Exception {
-        doTest(SubclassDenyAllServlet.class.getName(),
-                false, false, false,false);
+        doTest(SubclassDenyAllServlet.class.getName(), false, false, false, false);
     }
 
     @Test
     public void testSecurityAnnotationsSubclass2() throws Exception {
-        doTest(SubclassAllowAllServlet.class.getName(),
-                false, false, true, false);
+        doTest(SubclassAllowAllServlet.class.getName(), false, false, true, false);
     }
 
     @Test
     public void testSecurityAnnotationsMethods1() throws Exception {
-        doTest(MethodConstraintServlet.class.getName(),
-                false, false, false, false);
+        doTest(MethodConstraintServlet.class.getName(), false, false, false, false);
     }
 
     @Test
     public void testSecurityAnnotationsMethods2() throws Exception {
-        doTest(MethodConstraintServlet.class.getName(),
-                true, false, true, false);
+        doTest(MethodConstraintServlet.class.getName(), true, false, true, false);
     }
 
     @Test
@@ -159,9 +151,8 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         ByteChunk bc = new ByteChunk();
         int rc;
-        rc = getUrl("http://localhost:" + getPort() +
-                "/testStandardWrapper/securityAnnotationsWebXmlPriority",
-                bc, null, null);
+        rc = getUrl("http://localhost:" + getPort() + "/testStandardWrapper/securityAnnotationsWebXmlPriority", bc,
+                null, null);
 
         Assert.assertTrue(bc.getLength() > 0);
         Assert.assertEquals(403, rc);
@@ -173,8 +164,7 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         ByteChunk bc = new ByteChunk();
         int rc;
-        rc = getUrl("http://localhost:" + getPort() +
-                "/test/testStandardWrapper/securityAnnotationsMetaDataPriority",
+        rc = getUrl("http://localhost:" + getPort() + "/test/testStandardWrapper/securityAnnotationsMetaDataPriority",
                 bc, null, null);
 
         Assert.assertEquals("OK", bc.toString());
@@ -203,8 +193,7 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         ByteChunk bc = new ByteChunk();
         int rc;
-        rc = getUrl("http://localhost:" + getPort() + "/",
-                bc, null, null);
+        rc = getUrl("http://localhost:" + getPort() + "/", bc, null, null);
 
         Assert.assertTrue(bc.getLength() > 0);
         Assert.assertEquals(403, rc);
@@ -222,16 +211,14 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         ByteChunk bc = new ByteChunk();
         int rc;
-        rc = getUrl("http://localhost:" + getPort() + "/protected.jsp",
-                bc, null, null);
+        rc = getUrl("http://localhost:" + getPort() + "/protected.jsp", bc, null, null);
 
         Assert.assertTrue(bc.getLength() > 0);
         Assert.assertEquals(403, rc);
 
         bc.recycle();
 
-        rc = getUrl("http://localhost:" + getPort() + "/unprotected.jsp",
-                bc, null, null);
+        rc = getUrl("http://localhost:" + getPort() + "/unprotected.jsp", bc, null, null);
 
         Assert.assertEquals(200, rc);
         Assert.assertTrue(bc.toString().contains("00-OK"));
@@ -252,13 +239,12 @@ public class TestStandardWrapper extends TomcatBaseTest {
         doTestRoleMapping("context");
     }
 
-    private void doTestRoleMapping(String realmContainer)
-            throws Exception {
+    private void doTestRoleMapping(String realmContainer) throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
         ctx.addRoleMapping("testRole", "very-complex-role-name");
 
         Wrapper wrapper = Tomcat.addServlet(ctx, "servlet", RoleAllowServlet.class.getName());
@@ -272,8 +258,9 @@ public class TestStandardWrapper extends TomcatBaseTest {
         ch.setAlgorithm("SHA");
         realm.setCredentialHandler(ch);
 
-        /* Attach the realm to the appropriate container, but role mapping must
-         * always succeed because it is evaluated at context level.
+        /*
+         * Attach the realm to the appropriate container, but role mapping must always succeed because it is evaluated
+         * at context level.
          */
         if (realmContainer.equals("engine")) {
             tomcat.getEngine().setRealm(realm);
@@ -306,27 +293,25 @@ public class TestStandardWrapper extends TomcatBaseTest {
         // This now tests RealmBase#hasResourcePermission() because we need a wrapper
         // to be passed from an authenticator
         ByteChunk bc = new ByteChunk();
-        Map<String, List<String>> reqHeaders = new HashMap<>();
+        Map<String,List<String>> reqHeaders = new HashMap<>();
         List<String> authHeaders = new ArrayList<>();
         // testUser, testPwd
         authHeaders.add("Basic dGVzdFVzZXI6dGVzdFB3ZA==");
         reqHeaders.put("Authorization", authHeaders);
 
-        int rc = getUrl("http://localhost:" + getPort() + "/", bc, reqHeaders,
-                null);
+        int rc = getUrl("http://localhost:" + getPort() + "/", bc, reqHeaders, null);
 
         Assert.assertEquals("OK", bc.toString());
         Assert.assertEquals(200, rc);
     }
 
-    private void doTestSecurityAnnotationsAddServlet(boolean useCreateServlet)
-            throws Exception {
+    private void doTestSecurityAnnotationsAddServlet(boolean useCreateServlet) throws Exception {
 
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         Servlet s = new DenyAllServlet();
         ServletContainerInitializer sci = new SCI(s, useCreateServlet);
@@ -347,15 +332,14 @@ public class TestStandardWrapper extends TomcatBaseTest {
         }
     }
 
-    private void doTest(String servletClassName, boolean usePost,
-            boolean useRole, boolean expect200, boolean denyUncovered)
-            throws Exception {
+    private void doTest(String servletClassName, boolean usePost, boolean useRole, boolean expect200,
+            boolean denyUncovered) throws Exception {
 
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
 
         // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
+        Context ctx = getProgrammaticRootContext();
 
         ctx.setDenyUncoveredHttpMethods(denyUncovered);
 
@@ -387,11 +371,9 @@ public class TestStandardWrapper extends TomcatBaseTest {
 
         int rc;
         if (usePost) {
-            rc = postUrl(null, "http://localhost:" + getPort() + "/", bc,
-                    reqHeaders, null);
+            rc = postUrl(null, "http://localhost:" + getPort() + "/", bc, reqHeaders, null);
         } else {
-            rc = getUrl("http://localhost:" + getPort() + "/", bc, reqHeaders,
-                    null);
+            rc = getUrl("http://localhost:" + getPort() + "/", bc, reqHeaders, null);
         }
 
         if (expect200) {
@@ -407,16 +389,14 @@ public class TestStandardWrapper extends TomcatBaseTest {
         private static final long serialVersionUID = 1L;
 
         @Override
-        protected void doGet(HttpServletRequest req, HttpServletResponse resp)
-                throws ServletException, IOException {
+        protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
 
             resp.setContentType("text/plain");
             resp.getWriter().print("OK");
         }
 
         @Override
-        protected void doPost(HttpServletRequest req, HttpServletResponse resp)
-                throws ServletException, IOException {
+        protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
             doGet(req, resp);
         }
     }
@@ -435,20 +415,13 @@ public class TestStandardWrapper extends TomcatBaseTest {
         private static final long serialVersionUID = 1L;
     }
 
-    @ServletSecurity(value= @HttpConstraint(EmptyRoleSemantic.PERMIT),
-        httpMethodConstraints = {
-            @HttpMethodConstraint(value="GET",
-                    emptyRoleSemantic = EmptyRoleSemantic.DENY)
-        }
-    )
+    @ServletSecurity(value = @HttpConstraint(EmptyRoleSemantic.PERMIT), httpMethodConstraints = {
+            @HttpMethodConstraint(value = Method.GET, emptyRoleSemantic = EmptyRoleSemantic.DENY) })
     public static class MethodConstraintServlet extends TestServlet {
         private static final long serialVersionUID = 1L;
     }
 
-    @ServletSecurity(httpMethodConstraints = {
-            @HttpMethodConstraint(value="POST",rolesAllowed = "testRole")
-        }
-    )
+    @ServletSecurity(httpMethodConstraints = { @HttpMethodConstraint(value = Method.POST, rolesAllowed = "testRole") })
     public static class UncoveredGetServlet extends TestServlet {
         private static final long serialVersionUID = 1L;
     }
@@ -474,8 +447,7 @@ public class TestStandardWrapper extends TomcatBaseTest {
         }
 
         @Override
-        public void onStartup(Set<Class<?>> c, ServletContext ctx)
-                throws ServletException {
+        public void onStartup(Set<Class<?>> c, ServletContext ctx) throws ServletException {
             Servlet s;
 
             if (createServlet) {
@@ -487,177 +459,4 @@ public class TestStandardWrapper extends TomcatBaseTest {
             r.addMapping("/");
         }
     }
-
-
-    public static final int BUG51445_THREAD_COUNT = 5;
-
-    public static CountDownLatch latch = null;
-    public static final AtomicInteger counter = new AtomicInteger(0);
-
-    public static void initLatch() {
-        latch = new CountDownLatch(BUG51445_THREAD_COUNT);
-    }
-
-    @Test
-    public void testBug51445AddServlet() throws Exception {
-
-        initLatch();
-
-        Tomcat tomcat = getTomcatInstance();
-
-        // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
-
-        Tomcat.addServlet(ctx, "Bug51445", new Bug51445Servlet());
-        ctx.addServletMappingDecoded("/", "Bug51445");
-
-        tomcat.start();
-
-        // Start the threads
-        Bug51445Thread[] threads = new Bug51445Thread[5];
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
-            threads[i] = new Bug51445Thread(getPort());
-            threads[i].start();
-        }
-
-        // Wait for threads to finish
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
-            threads[i].join();
-        }
-
-        Set<String> servlets = new HashSet<>();
-        // Output the result
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
-            System.out.println(threads[i].getResult());
-        }
-
-        // Check the result
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
-            String[] results = threads[i].getResult().split(",");
-            Assert.assertEquals(2, results.length);
-            Assert.assertEquals("10", results[0]);
-            Assert.assertFalse(servlets.contains(results[1]));
-            servlets.add(results[1]);
-        }
-    }
-
-    @Test
-    public void testBug51445AddChild() throws Exception {
-
-        initLatch();
-
-        Tomcat tomcat = getTomcatInstance();
-
-        // No file system docBase required
-        Context ctx = tomcat.addContext("", null);
-
-        StandardWrapper wrapper = new StandardWrapper();
-        wrapper.setServletName("Bug51445");
-        wrapper.setServletClass(Bug51445Servlet.class.getName());
-        ctx.addChild(wrapper);
-        ctx.addServletMappingDecoded("/", "Bug51445");
-
-        tomcat.start();
-
-        // Start the threads
-        Bug51445Thread[] threads = new Bug51445Thread[5];
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
-            threads[i] = new Bug51445Thread(getPort());
-            threads[i].start();
-        }
-
-        // Wait for threads to finish
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
-            threads[i].join();
-        }
-
-        Set<String> servlets = new HashSet<>();
-        // Output the result
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
-            System.out.println(threads[i].getResult());
-        }
-        // Check the result
-        for (int i = 0; i < BUG51445_THREAD_COUNT; i ++) {
-            String[] results = threads[i].getResult().split(",");
-            Assert.assertEquals(2, results.length);
-            Assert.assertEquals("10", results[0]);
-            Assert.assertFalse(servlets.contains(results[1]));
-            servlets.add(results[1]);
-        }
-    }
-
-    private static class Bug51445Thread extends Thread {
-
-        private int port;
-        private String result;
-
-        public Bug51445Thread(int port) {
-            this.port = port;
-        }
-
-        @Override
-        public void run() {
-            try {
-                ByteChunk res = getUrl("http://localhost:" + port + "/");
-                result = res.toString();
-            } catch (IOException ioe) {
-                result = ioe.getMessage();
-            }
-        }
-
-        public String getResult() {
-            return result;
-        }
-    }
-
-    /**
-     * SingleThreadModel servlet that sets a value in the init() method.
-     */
-    @SuppressWarnings("deprecation")
-    public static class Bug51445Servlet extends HttpServlet
-            implements jakarta.servlet.SingleThreadModel {
-
-        private static final long serialVersionUID = 1L;
-        private static final long LATCH_TIMEOUT = 60;
-
-        private int data = 0;
-        private int counter;
-
-        public Bug51445Servlet() {
-            // Use this rather than hashCode since in some environments,
-            // multiple instances of this object were created with the same
-            // hashCode
-            counter = TestStandardWrapper.counter.incrementAndGet();
-        }
-
-        @Override
-        protected void doGet(HttpServletRequest req, HttpServletResponse resp)
-                throws ServletException, IOException {
-
-            boolean latchAwaitResult = false;
-
-            // Ensure all threads have their own instance of the servlet
-            latch.countDown();
-            try {
-                latchAwaitResult = latch.await(LATCH_TIMEOUT, TimeUnit.SECONDS);
-            } catch (InterruptedException e) {
-                // Ignore
-            }
-
-            resp.setContentType("text/plain");
-            if (latchAwaitResult) {
-                resp.getWriter().print(data);
-            } else {
-                resp.getWriter().print("Latch await failed");
-            }
-            resp.getWriter().print(",");
-            resp.getWriter().print(counter);
-        }
-
-        @Override
-        public void init(ServletConfig config) throws ServletException {
-            super.init(config);
-            data = 10;
-        }
-    }
 }
